@@ -149,8 +149,7 @@ _kdc_find_etype (
 	const hdb_entry_ex */*princ*/,
 	krb5_enctype */*etypes*/,
 	unsigned /*len*/,
-	Key **/*ret_key*/,
-	krb5_enctype */*ret_etype*/);
+	Key **/*ret_key*/);
 
 const PA_DATA*
 _kdc_find_padata (
@@ -223,7 +222,8 @@ _kdc_pac_verify (
 	const krb5_principal /*client_principal*/,
 	hdb_entry_ex */*client*/,
 	hdb_entry_ex */*server*/,
-	krb5_pac */*pac*/);
+	krb5_pac */*pac*/,
+	int */*verified*/);
 
 krb5_error_code
 _kdc_pk_check_client (
@@ -239,15 +239,6 @@ _kdc_pk_free_client_param (
 	krb5_context /*context*/,
 	pk_client_params */*cp*/);
 
-krb5_error_code
-_kdc_pk_initialize (
-	krb5_context /*context*/,
-	krb5_kdc_configuration */*config*/,
-	const char */*user_id*/,
-	const char */*anchors*/,
-	char **/*pool*/,
-	char **/*revoke_list*/);
-
 krb5_error_code
 _kdc_pk_mk_pa_reply (
 	krb5_context /*context*/,
