@@ -17,6 +17,12 @@ _hdb_find_master_key (
 	uint32_t */*mkvno*/,
 	hdb_master_key /*mkey*/);
 
+krb5_error_code
+_hdb_keytab2hdb_entry (
+	krb5_context /*context*/,
+	const krb5_keytab_entry */*ktentry*/,
+	hdb_entry_ex */*entry*/);
+
 int
 _hdb_mkey_decrypt (
 	krb5_context /*context*/,
