@@ -9,15 +9,18 @@ extern "C" {
 #endif
 
 #ifndef HX509_LIB
+#ifndef HX509_LIB_FUNCTION
 #if defined(_WIN32)
-#define HX509_LIB_FUNCTION __stdcall __declspec(dllimport)
+#define HX509_LIB_FUNCTION __declspec(dllimport)
+#define HX509_LIB_CALL __stdcall
 #define HX509_LIB_VARIABLE __declspec(dllimport)
 #else
 #define HX509_LIB_FUNCTION
+#define HX509_LIB_CALL
 #define HX509_LIB_VARIABLE
 #endif
 #endif
-
+#endif
 void
 hx509_bitstring_print (
 	const heim_bit_string */*b*/,
@@ -156,6 +159,13 @@ hx509_ca_tbs_set_template (
 	int /*flags*/,
 	hx509_cert /*cert*/);
 
+int
+hx509_ca_tbs_set_unique (
+	hx509_context /*context*/,
+	hx509_ca_tbs /*tbs*/,
+	const heim_bit_string */*subjectUniqueID*/,
+	const heim_bit_string */*issuerUniqueID*/);
+
 int
 hx509_ca_tbs_subject_expand (
 	hx509_context /*context*/,
@@ -224,6 +234,12 @@ hx509_cert_get_issuer (
 	hx509_cert /*p*/,
 	hx509_name */*name*/);
 
+int
+hx509_cert_get_issuer_unique_id (
+	hx509_context /*context*/,
+	hx509_cert /*p*/,
+	heim_bit_string */*issuer*/);
+
 time_t
 hx509_cert_get_notAfter (hx509_cert /*p*/);
 
@@ -240,6 +256,12 @@ hx509_cert_get_subject (
 	hx509_cert /*p*/,
 	hx509_name */*name*/);
 
+int
+hx509_cert_get_subject_unique_id (
+	hx509_context /*context*/,
+	hx509_cert /*p*/,
+	heim_bit_string */*subject*/);
+
 int
 hx509_cert_have_private_key (hx509_cert /*p*/);
 
@@ -321,8 +343,16 @@ hx509_certs_init (
 	hx509_lock /*lock*/,
 	hx509_certs */*certs*/);
 
+#ifdef __BLOCKS__
 int
 hx509_certs_iter (
+	hx509_context /*context*/,
+	hx509_certs /*certs*/,
+	int (^func)(hx509_cert));
+#endif /* __BLOCKS__ */
+
+int
+hx509_certs_iter_f (
 	hx509_context /*context*/,
 	hx509_certs /*certs*/,
 	int (*/*func*/)(hx509_context, void *, hx509_cert),
@@ -975,9 +1005,6 @@ hx509_signature_ecdsa_with_sha1 (void);
 const AlgorithmIdentifier *
 hx509_signature_ecdsa_with_sha256 (void);
 
-const AlgorithmIdentifier *
-hx509_signature_md2 (void);
-
 const AlgorithmIdentifier *
 hx509_signature_md5 (void);
 
@@ -987,9 +1014,6 @@ hx509_signature_rsa (void);
 const AlgorithmIdentifier *
 hx509_signature_rsa_pkcs1_x509 (void);
 
-const AlgorithmIdentifier *
-hx509_signature_rsa_with_md2 (void);
-
 const AlgorithmIdentifier *
 hx509_signature_rsa_with_md5 (void);
 
