@@ -4,14 +4,25 @@
 
 #include <stdarg.h>
 
-void KRB5_LIB_FUNCTION
-_krb5_aes_cts_encrypt (
-	const unsigned char */*in*/,
-	unsigned char */*out*/,
-	size_t /*len*/,
-	const AES_KEY */*key*/,
-	unsigned char */*ivec*/,
-	const int /*encryptp*/);
+#if !defined(__GNUC__) && !defined(__attribute__)
+#define __attribute__(x)
+#endif
+
+void
+_heim_krb5_ipc_client_clear_target (void);
+
+void
+_heim_krb5_ipc_client_set_target_uid (uid_t /*uid*/);
+
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
+_krb5_build_authenticator (
+	krb5_context /*context*/,
+	krb5_auth_context /*auth_context*/,
+	krb5_enctype /*enctype*/,
+	krb5_creds */*cred*/,
+	Checksum */*cksum*/,
+	krb5_data */*result*/,
+	krb5_key_usage /*usage*/);
 
 krb5_error_code
 _krb5_cc_allocate (
@@ -19,6 +30,47 @@ _krb5_cc_allocate (
 	const krb5_cc_ops */*ops*/,
 	krb5_ccache */*id*/);
 
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
+_krb5_config_copy (
+	krb5_context /*context*/,
+	krb5_config_section */*c*/,
+	krb5_config_section **/*head*/);
+
+KRB5_LIB_FUNCTION const void * KRB5_LIB_CALL
+_krb5_config_get (
+	krb5_context /*context*/,
+	const krb5_config_section */*c*/,
+	int /*type*/,
+	...);
+
+KRB5_LIB_FUNCTION const void * KRB5_LIB_CALL
+_krb5_config_get_next (
+	krb5_context /*context*/,
+	const krb5_config_section */*c*/,
+	const krb5_config_binding **/*pointer*/,
+	int /*type*/,
+	...);
+
+const void *
+_krb5_config_vget (
+	krb5_context /*context*/,
+	const krb5_config_section */*c*/,
+	int /*type*/,
+	va_list /*args*/);
+
+KRB5_LIB_FUNCTION const void * KRB5_LIB_CALL
+_krb5_config_vget_next (
+	krb5_context /*context*/,
+	const krb5_config_section */*c*/,
+	const krb5_config_binding **/*pointer*/,
+	int /*type*/,
+	va_list /*args*/);
+
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
+_krb5_copy_send_to_kdc_func (
+	krb5_context /*context*/,
+	krb5_context /*to*/);
+
 void
 _krb5_crc_init_table (void);
 
@@ -28,6 +80,14 @@ _krb5_crc_update (
 	size_t /*len*/,
 	uint32_t /*res*/);
 
+void KRB5_LIB_FUNCTION
+_krb5_debug (
+	krb5_context /*context*/,
+	int /*level*/,
+	const char */*fmt*/,
+	...)
+     __attribute__((format (printf, 3, 4)));
+
 krb5_error_code
 _krb5_dh_group_ok (
 	krb5_context /*context*/,
@@ -38,11 +98,10 @@ _krb5_dh_group_ok (
 	struct krb5_dh_moduli **/*moduli*/,
 	char **/*name*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
-_krb5_enctype_to_oid (
+krb5_error_code
+_krb5_erase_file (
 	krb5_context /*context*/,
-	krb5_enctype /*etype*/,
-	heim_oid */*oid*/);
+	const char */*filename*/);
 
 krb5_error_code
 _krb5_expand_default_cc_name (
@@ -50,6 +109,12 @@ _krb5_expand_default_cc_name (
 	const char */*str*/,
 	char **/*res*/);
 
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
+_krb5_expand_path_tokens (
+	krb5_context /*context*/,
+	const char */*path_in*/,
+	char **/*ppath_out*/);
+
 int
 _krb5_extract_ticket (
 	krb5_context /*context*/,
@@ -70,37 +135,39 @@ _krb5_free_krbhst_info (krb5_krbhst_info */*hi*/);
 void
 _krb5_free_moduli (struct krb5_dh_moduli **/*moduli*/);
 
+krb5_error_code
+_krb5_get_cred_kdc_any (
+	krb5_context /*context*/,
+	krb5_kdc_flags /*flags*/,
+	krb5_ccache /*ccache*/,
+	krb5_creds */*in_creds*/,
+	krb5_principal /*impersonate_principal*/,
+	Ticket */*second_ticket*/,
+	krb5_creds **/*out_creds*/,
+	krb5_creds ***/*ret_tgts*/);
+
 krb5_error_code
 _krb5_get_default_principal_local (
 	krb5_context /*context*/,
 	krb5_principal */*princ*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_get_host_realm_int (
 	krb5_context /*context*/,
 	const char */*host*/,
 	krb5_boolean /*use_dns*/,
 	krb5_realm **/*realms*/);
 
-krb5_error_code
-_krb5_get_init_creds_opt_copy (
-	krb5_context /*context*/,
-	const krb5_get_init_creds_opt */*in*/,
-	krb5_get_init_creds_opt **/*out*/);
-
-void KRB5_LIB_FUNCTION
-_krb5_get_init_creds_opt_free_krb5_error (krb5_get_init_creds_opt */*opt*/);
-
-void KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION void KRB5_LIB_CALL
 _krb5_get_init_creds_opt_free_pkinit (krb5_get_init_creds_opt */*opt*/);
 
-void KRB5_LIB_FUNCTION
-_krb5_get_init_creds_opt_set_krb5_error (
+krb5_error_code KRB5_LIB_FUNCTION
+_krb5_get_init_creds_opt_set_pkinit_user_certs (
 	krb5_context /*context*/,
 	krb5_get_init_creds_opt */*opt*/,
-	const KRB_ERROR */*error*/);
+	struct hx509_certs_data */*certs*/);
 
-krb5_ssize_t KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_ssize_t KRB5_LIB_CALL
 _krb5_get_int (
 	void */*buffer*/,
 	unsigned long */*value*/,
@@ -113,18 +180,13 @@ _krb5_get_krbtgt (
 	krb5_realm /*realm*/,
 	krb5_creds **/*cred*/);
 
-krb5_error_code
-_krb5_kcm_chmod (
+krb5_boolean KRB5_LIB_FUNCTION
+_krb5_have_debug (
 	krb5_context /*context*/,
-	krb5_ccache /*id*/,
-	uint16_t /*mode*/);
+	int /*level*/);
 
-krb5_error_code
-_krb5_kcm_chown (
-	krb5_context /*context*/,
-	krb5_ccache /*id*/,
-	uint32_t /*uid*/,
-	uint32_t /*gid*/);
+krb5_boolean
+_krb5_homedir_access (krb5_context /*context*/);
 
 krb5_error_code
 _krb5_kcm_get_initial_ticket (
@@ -149,7 +211,7 @@ _krb5_kcm_noop (
 	krb5_context /*context*/,
 	krb5_ccache /*id*/);
 
-krb5_error_code
+krb5_error_code KRB5_CALLCONV
 _krb5_kdc_retry (
 	krb5_context /*context*/,
 	krb5_sendto_ctx /*ctx*/,
@@ -157,7 +219,7 @@ _krb5_kdc_retry (
 	const krb5_data */*reply*/,
 	int */*action*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_krb_cr_err_reply (
 	krb5_context /*context*/,
 	const char */*name*/,
@@ -168,7 +230,7 @@ _krb5_krb_cr_err_reply (
 	const char */*e_string*/,
 	krb5_data */*data*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_krb_create_auth_reply (
 	krb5_context /*context*/,
 	const char */*pname*/,
@@ -181,7 +243,7 @@ _krb5_krb_create_auth_reply (
 	const krb5_data */*cipher*/,
 	krb5_data */*data*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_krb_create_ciph (
 	krb5_context /*context*/,
 	const krb5_keyblock */*session*/,
@@ -195,7 +257,7 @@ _krb5_krb_create_ciph (
 	const krb5_keyblock */*key*/,
 	krb5_data */*enc_data*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_krb_create_ticket (
 	krb5_context /*context*/,
 	unsigned char /*flags*/,
@@ -211,7 +273,7 @@ _krb5_krb_create_ticket (
 	const krb5_keyblock */*key*/,
 	krb5_data */*enc_data*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_krb_decomp_ticket (
 	krb5_context /*context*/,
 	const krb5_data */*enc_ticket*/,
@@ -221,22 +283,22 @@ _krb5_krb_decomp_ticket (
 	char **/*sinstance*/,
 	struct _krb5_krb_auth_data */*ad*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_krb_dest_tkt (
 	krb5_context /*context*/,
 	const char */*tkfile*/);
 
-void KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION void KRB5_LIB_CALL
 _krb5_krb_free_auth_data (
 	krb5_context /*context*/,
 	struct _krb5_krb_auth_data */*ad*/);
 
-time_t KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION time_t KRB5_LIB_CALL
 _krb5_krb_life_to_time (
 	int /*start*/,
 	int /*life_*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_krb_rd_req (
 	krb5_context /*context*/,
 	krb5_data */*authent*/,
@@ -247,14 +309,14 @@ _krb5_krb_rd_req (
 	const krb5_keyblock */*key*/,
 	struct _krb5_krb_auth_data */*ad*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_krb_tf_setup (
 	krb5_context /*context*/,
 	struct credentials */*v4creds*/,
 	const char */*tkfile*/,
 	int /*append*/);
 
-int KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION int KRB5_LIB_CALL
 _krb5_krb_time_to_life (
 	time_t /*start*/,
 	time_t /*end*/);
@@ -265,6 +327,18 @@ _krb5_krbhost_info_move (
 	krb5_krbhst_info */*from*/,
 	krb5_krbhst_info **/*to*/);
 
+const char *
+_krb5_krbhst_get_realm (krb5_krbhst_handle /*handle*/);
+
+krb5_error_code
+_krb5_kt_principal_not_found (
+	krb5_context /*context*/,
+	krb5_error_code /*ret*/,
+	krb5_keytab /*id*/,
+	krb5_const_principal /*principal*/,
+	krb5_enctype /*enctype*/,
+	int /*kvno*/);
+
 krb5_error_code
 _krb5_mk_req_internal (
 	krb5_context /*context*/,
@@ -276,19 +350,13 @@ _krb5_mk_req_internal (
 	krb5_key_usage /*checksum_usage*/,
 	krb5_key_usage /*encrypt_usage*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_n_fold (
 	const void */*str*/,
 	size_t /*len*/,
 	void */*key*/,
 	size_t /*size*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
-_krb5_oid_to_enctype (
-	krb5_context /*context*/,
-	const heim_oid */*oid*/,
-	krb5_enctype */*etype*/);
-
 krb5_error_code
 _krb5_pac_sign (
 	krb5_context /*context*/,
@@ -313,14 +381,17 @@ _krb5_parse_moduli_line (
 	char */*p*/,
 	struct krb5_dh_moduli **/*m*/);
 
-void KRB5_LIB_FUNCTION
-_krb5_pk_allow_proxy_certificate (
-	struct krb5_pk_identity */*id*/,
-	int /*boolean*/);
-
-void KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION void KRB5_LIB_CALL
 _krb5_pk_cert_free (struct krb5_pk_cert */*cert*/);
 
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
+_krb5_pk_enterprise_cert (
+	krb5_context /*context*/,
+	const char */*user_id*/,
+	krb5_const_realm /*realm*/,
+	krb5_principal */*principal*/,
+	struct hx509_certs_data **/*res*/);
+
 krb5_error_code
 _krb5_pk_kdf (
 	krb5_context /*context*/,
@@ -335,7 +406,7 @@ _krb5_pk_kdf (
 	const Ticket */*ticket*/,
 	krb5_keyblock */*key*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_pk_load_id (
 	krb5_context /*context*/,
 	struct krb5_pk_identity **/*ret_id*/,
@@ -347,17 +418,19 @@ _krb5_pk_load_id (
 	void */*prompter_data*/,
 	char */*password*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_pk_mk_ContentInfo (
 	krb5_context /*context*/,
 	const krb5_data */*buf*/,
 	const heim_oid */*oid*/,
 	struct ContentInfo */*content_info*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_pk_mk_padata (
 	krb5_context /*context*/,
 	void */*c*/,
+	int /*ic_flags*/,
+	int /*win2k*/,
 	const KDC_REQ_BODY */*req_body*/,
 	unsigned /*nonce*/,
 	METHOD_DATA */*md*/);
@@ -372,7 +445,7 @@ _krb5_pk_octetstring2key (
 	const heim_octet_string */*k_n*/,
 	krb5_keyblock */*key*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_pk_rd_pa_reply (
 	krb5_context /*context*/,
 	const char */*realm*/,
@@ -384,16 +457,6 @@ _krb5_pk_rd_pa_reply (
 	PA_DATA */*pa*/,
 	krb5_keyblock **/*key*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
-_krb5_pk_verify_sign (
-	krb5_context /*context*/,
-	const void */*data*/,
-	size_t /*length*/,
-	struct krb5_pk_identity */*id*/,
-	heim_oid */*contentType*/,
-	krb5_data */*content*/,
-	struct krb5_pk_cert **/*signer*/);
-
 krb5_error_code
 _krb5_plugin_find (
 	krb5_context /*context*/,
@@ -410,36 +473,31 @@ _krb5_plugin_get_next (struct krb5_plugin */*p*/);
 void *
 _krb5_plugin_get_symbol (struct krb5_plugin */*p*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_principal2principalname (
 	PrincipalName */*p*/,
 	const krb5_principal /*from*/);
 
-krb5_boolean KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_boolean KRB5_LIB_CALL
 _krb5_principal_compare_PrincipalName (
 	krb5_context /*context*/,
 	krb5_const_principal /*princ1*/,
 	PrincipalName */*princ2*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_principalname2krb5_principal (
 	krb5_context /*context*/,
 	krb5_principal */*principal*/,
 	const PrincipalName /*from*/,
 	const Realm /*realm*/);
 
-krb5_ssize_t KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_ssize_t KRB5_LIB_CALL
 _krb5_put_int (
 	void */*buffer*/,
 	unsigned long /*value*/,
 	size_t /*size*/);
 
-krb5_error_code KRB5_LIB_FUNCTION
-_krb5_rd_req_out_ctx_alloc (
-	krb5_context /*context*/,
-	krb5_rd_req_out_ctx */*ctx*/);
-
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 _krb5_s4u2self_to_checksumdata (
 	krb5_context /*context*/,
 	const PA_S4U2Self */*self*/,
@@ -447,7 +505,7 @@ _krb5_s4u2self_to_checksumdata (
 
 int
 _krb5_send_and_recv_tcp (
-	int /*fd*/,
+	krb5_socket_t /*fd*/,
 	time_t /*tmout*/,
 	const krb5_data */*req*/,
 	krb5_data */*rep*/);
